/***
 * Excerpted from "Rails For Java Programmers"
 * We make no guarantees that this code is fit for any purpose. 
 * Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
***/
package com.relevancellc.people.service;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.relevancellc.people.model.User;
import com.relevancellc.people.service.UserExistsException;
import com.relevancellc.people.service.UserManager;
import org.springframework.remoting.jaxrpc.ServletEndpointSupport;

/**
 * Example Webservice Proxy-Facade for UserService
 * @author Mika Gckel <mika at cybercon dot de>
 * @author RanHow Chen
 * 
 * @axisfuse.service
 * 		name="UserService" 
 * 		scope="Request"
 * 		enable-remote-admin="false"
 * 		namespace="http://service.appfuse.org/"
 * 		provider="java:RPC"
 * 		style="rpc"
 * 		use="encoded"
 * 		include-all="true"
 */
public class UserEndpoint extends ServletEndpointSupport {
	
    protected final Log log = LogFactory.getLog(getClass());
	
	private UserManager userManager;

	protected void onInit() {
		log.debug("onInit()");
        this.userManager = (UserManager) getWebApplicationContext().getBean("userManager");
    }
	
	public User getUser(String username) {
		log.debug("getUser("+username+")");
		return userManager.getUser(username);
	}

	public User [] getUsers(User user) {
		List users = userManager.getUsers(user); 
		return users == null ? null : (User[]) users.toArray(new User[users.size()]);
	}

	public void removeUser(String username) {
		userManager.removeUser(username);
	}

	public void saveUser(User user) throws UserExistsException {
		userManager.saveUser(user);
	}

}
