#---
# Excerpted from "Rails For Java Programmers"
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/fr_r4j for more book information.
#---
require File.dirname(__FILE__) + '/../test_helper'
require 'people_controller'

# Re-raise errors caught by the controller.
class PeopleController; def rescue_action(e) raise e end; end

class PeopleControllerTest < Test::Unit::TestCase
  include FlexMock::TestCase
  fixtures :people

  def setup
    @controller = PeopleController.new
    @request    = ActionController::TestRequest.new
    @response   = ActionController::TestResponse.new
    @valid_args = {:first_name=>'test', :last_name=>'person'}
  end
  
  def test_index
    get :index
    assert_response :success
    assert_template 'list'
  end

  def test_list
    get :list

    assert_response :success
    assert_template 'list'

    assert_not_nil assigns(:people)
  end
  
  def test_list_with_search
    get :list, :search=>'Gehtland'

    assert_response :success
    assert_template 'list'

    assert_equal [people(:another)], assigns(:people)
  end

  def test_show
    get :show, :id => 1

    assert_response :success
    assert_template 'show'

    assert_not_nil assigns(:person)
    assert assigns(:person).valid?
  end

  def test_new
    get :new

    assert_response :success
    assert_template 'new'

    assert_not_nil assigns(:person)
  end

  
  def test_create_succeeds
    flexstub(Person).should_receive(:new).and_return { 
      flexmock('person') do |m|                        
        m.should_receive(:save).and_return(true)       
      end
    }
    post :create
    assert_response :redirect
    assert_redirected_to :action => 'list'
  end
  

  
  def test_create_fails
    flexstub(Person).should_receive(:new).and_return {
      errs = flexmock('errs') do |m|
        m.should_ignore_missing
      end
      inst = flexmock('person') do |m|
        m.should_ignore_missing
        m.should_receive(:errors).and_return(errs)
        m.should_receive(:save).and_return(false)
      end
    }
    post :create
    assert_response :success
    assert_template 'new'
  end
  
  
  def test_edit
    get :edit, :id => 1

    assert_response :success
    assert_template 'edit'

    assert_not_nil assigns(:person)
    assert assigns(:person).valid?
  end

  def test_update
    post :update, :id=>1, :person => @valid_args
    assert_response :redirect
    assert_redirected_to :action => 'show', :id => 1
  end
  
  def test_update_fails
    post :update, :id=>1, :person=>{:first_name=>nil}
    assert_response :success
    assert_template 'edit'
  end

  def test_destroy
    assert_not_nil Person.find(1)

    post :destroy, :id => 1
    assert_response :redirect
    assert_redirected_to :action => 'list'

    assert_raise(ActiveRecord::RecordNotFound) {
      Person.find(1)
    }
  end
end
